"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cdk_1 = require("@aws-cdk/cdk");
const aws_apigateway_1 = require("@aws-cdk/aws-apigateway");
const CM = require("@aws-cdk/aws-certificatemanager");
const CF = require("@aws-cdk/aws-cloudfront");
const CB = require("@aws-cdk/aws-codebuild");
const aws_codepipeline_1 = require("@aws-cdk/aws-codepipeline");
const Events = require("@aws-cdk/aws-events");
const IAM = require("@aws-cdk/aws-iam");
const aws_lambda_1 = require("@aws-cdk/aws-lambda");
const aws_logs_1 = require("@aws-cdk/aws-logs");
const S3 = require("@aws-cdk/aws-s3");
const lib_1 = require("./lib");
class SeagullStack extends cdk_1.Stack {
    constructor(app, projectName, stackProps) {
        super(app, projectName, stackProps);
    }
    addLogGroup(logGroupName) {
        const retentionDays = Infinity;
        const props = { logGroupName, retentionDays };
        return new aws_logs_1.LogGroup(this, `${this.id}-${logGroupName}`, props);
    }
    addS3(bucketName, role) {
        const s3Props = { bucketName };
        const bucket = new S3.Bucket(this, `${this.id}-${bucketName}`, s3Props);
        role && bucket.grantReadWrite(role);
        return bucket;
    }
    addUniversalLambda(lambdaName, folder, role) {
        const name = `${this.id}-${lambdaName}`;
        const conf = {
            code: aws_lambda_1.Code.asset(`${folder}/.seagull/deploy`),
            description: 'universal route',
            environment: { MODE: 'cloud', APP: this.id },
            functionName: `${name}-handler`,
            handler: 'dist/assets/backend/lambda.handler',
            memorySize: 1536,
            role,
            runtime: aws_lambda_1.Runtime.NodeJS810,
            timeout: 300,
        };
        return new aws_lambda_1.Function(this, name, conf);
    }
    addUniversalApiGateway(apiGWName, lambda, stageName) {
        const name = `${this.id}-${apiGWName}`;
        const defaultIntegration = new aws_apigateway_1.LambdaIntegration(lambda);
        const conf = { binaryMediaTypes: ['*/*'], deployOptions: { stageName } };
        const apiGateway = new aws_apigateway_1.RestApi(this, name, conf);
        const proxy = apiGateway.root.addResource('{any+}');
        apiGateway.root.addMethod('GET', defaultIntegration);
        apiGateway.root.addMethod('POST', defaultIntegration);
        apiGateway.root.addMethod('DELETE', defaultIntegration);
        proxy.addMethod('GET', defaultIntegration);
        proxy.addMethod('POST', defaultIntegration);
        proxy.addMethod('DELETE', defaultIntegration);
        return apiGateway;
    }
    addIAMRole(roleName, principleName, actions) {
        const name = `${this.id}-${roleName}`;
        const roleParams = { assumedBy: new IAM.ServicePrincipal(principleName) };
        const role = new IAM.Role(this, name, roleParams);
        const policyStatement = new IAM.PolicyStatement();
        role.addToPolicy(policyStatement.addAllResources().addActions(...actions));
        return role;
    }
    addCloudfront(cfdName, props) {
        const name = `${this.id}-${cfdName}`;
        const domainName = lib_1.getApiGatewayDomain(props.apiGateway.url);
        const originPath = lib_1.getApiGatewayPath(props.apiGateway.url);
        const allowedMethods = CF.CloudFrontAllowedMethods.ALL;
        const forwardedValues = { headers: ['authorization'], queryString: true };
        const isDefaultBehavior = true;
        const behaviors = [{ allowedMethods, forwardedValues, isDefaultBehavior }];
        const customOriginSource = { domainName };
        const conf = {
            aliasConfiguration: props.aliasConfig,
            comment: this.id,
            defaultRootObject: '',
            originConfigs: [{ behaviors, customOriginSource, originPath }],
        };
        return new CF.CloudFrontWebDistribution(this, name, conf);
    }
    addPipeline(pipelineName) {
        const name = `${this.id}-${pipelineName}`;
        return new aws_codepipeline_1.Pipeline(this, name, { pipelineName: name });
    }
    addSourceStage(name, config) {
        const stageName = `${this.id}-stage-${name}`;
        const sourceName = `${this.id}-github-${name}`;
        const { atIndex, branch, owner, pipeline, repo, oauthToken } = config;
        const stage = pipeline.addStage(stageName, { placement: { atIndex } });
        const stageConfig = { branch, oauthToken, owner, repo, stage };
        return new aws_codepipeline_1.GitHubSourceAction(this, sourceName, stageConfig);
    }
    addBuildStage(name, config) {
        const stageName = `${this.id}-stage-${name}`;
        const buildName = `${this.id}-code-${name}`;
        const projectName = `${this.id}-project-${name}`;
        const { atIndex, build, env, install, pipeline, postBuild, role } = config;
        const buildImage = CB.LinuxBuildImage.UBUNTU_14_04_NODEJS_8_11_0;
        const phases = { build, install, post_build: postBuild };
        const projectConfig = {
            buildSpec: { env, phases, version: '0.2' },
            environment: { buildImage },
            role,
        };
        const project = new CB.PipelineProject(this, projectName, projectConfig);
        const stage = pipeline.addStage(stageName, { placement: { atIndex } });
        const stageConfig = { project, stage };
        return new CB.PipelineBuildAction(this, buildName, stageConfig);
    }
    addSecretParam(name, ssmParameter) {
        const secretConfig = { ssmParameter };
        return new cdk_1.SecretParameter(this, name, secretConfig);
    }
    addNewCert(name, domains) {
        const domainName = domains[0];
        const altNames = domains.slice(1);
        const subjectAlternativeNames = altNames.length > 0 ? altNames : undefined;
        const props = { domainName, subjectAlternativeNames };
        return new CM.Certificate(this, `${this.id}-${name}`, props);
    }
    importS3(bucketName, role) {
        const name = `${this.id}-${bucketName}`;
        const bucketArn = `arn:aws:s3:::${bucketName}`;
        const bucket = S3.Bucket.import(this, name, { bucketArn });
        role && bucket.grantReadWrite(role);
        return bucket;
    }
    addEventRule(rule, target) {
        const name = rule.path.split('/').pop() || 'rootPath';
        const schedule = {
            scheduleExpression: rule.cron,
        };
        const eventRule = new Events.EventRule(this, name, schedule);
        eventRule.addTarget(target, { jsonTemplate: `{"path":"${rule.path}"}` });
        return eventRule;
    }
}
exports.SeagullStack = SeagullStack;
//# sourceMappingURL=seagull_stack.js.map