"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commands_fs_1 = require("@seagull/commands-fs");
const aws_cdk_1 = require("aws-cdk");
require("ts-node/register");
const aws = require("../aws_sdk_handler");
const lib = require("../lib");
const provide_asset_folder_1 = require("../provide_asset_folder");
const seagull_app_1 = require("../seagull_app");
const set_aws_credentials_1 = require("../set_aws_credentials");
class SeagullProject {
    constructor(props) {
        this.appPath = props.appPath;
        this.branch = props.branch;
        this.mode = props.mode;
        this.profile = props.profile;
        this.region = props.region;
        this.pkgJson = require(`${this.appPath}/package.json`);
        const propsACMHandler = props.handlers && props.handlers.acmHandler;
        const propsCFHandler = props.handlers && props.handlers.cloudfrontHandler;
        const propsSTSHandler = props.handlers && props.handlers.stsHandler;
        this.acm = propsACMHandler || new aws.ACMHandler();
        this.cloudfront = propsCFHandler || new aws.CloudfrontHandler();
        this.sts = propsSTSHandler || new aws.STSHandler();
    }
    createSeagullApp() {
        return __awaiter(this, void 0, void 0, function* () {
            const name = this.getAppName();
            const sdk = new aws_cdk_1.SDK({});
            const account = yield sdk.defaultAccount();
            const itemBucketName = yield this.getBucketName();
            const actions = [
                'sts:AssumeRole',
                'logs:CreateLogStream',
                'logs:PutLogEvents',
                'lambda:InvokeFunction',
                'lambda:InvokeAsync',
                'ses:*',
                's3:*',
                'events:*',
            ];
            const aliasConfig = yield aws.checkForAliasConfig(this.pkgJson, this.acm);
            const appProps = {
                addAssets: true,
                appPath: this.appPath,
                itemsBucket: itemBucketName,
                projectName: name,
                stackProps: { env: { account, region: this.region } },
            };
            yield addResources(this.appPath, itemBucketName);
            const app = new seagull_app_1.SeagullApp(appProps);
            const role = app.stack.addIAMRole('role', 'lambda.amazonaws.com', actions);
            app.role = role;
            const lambda = app.stack.addUniversalLambda('lambda', this.appPath, role);
            const apiGW = app.stack.addUniversalApiGateway('apiGW', lambda, this.mode);
            app.stack.addCloudfront('cloudfront', { apiGateway: apiGW, aliasConfig });
            const s3DeploymentNeeded = this.mode === 'prod' || this.branch === 'master';
            const importS3 = () => app.stack.importS3(itemBucketName, role);
            const addS3 = () => app.stack.addS3(itemBucketName, role);
            s3DeploymentNeeded ? addS3() : importS3();
            app.stack.addLogGroup(`/aws/lambda/${name}-lambda-handler`);
            app.stack.addLogGroup(`/${name}/data-log`);
            const cronJson = yield buildCronJson(this.appPath);
            cronJson.forEach((rule) => {
                app.stack.addEventRule(rule, lambda);
            });
            return app;
        });
    }
    customizeStack(app) {
        return __awaiter(this, void 0, void 0, function* () {
            const extensionPath = `${this.appPath}/infrastructure-aws.ts`;
            const hasExtensionFile = yield new commands_fs_1.FS.Exists(extensionPath).execute();
            return hasExtensionFile && (yield Promise.resolve().then(() => require(`${extensionPath}`))).default(app);
        });
    }
    getAppName() {
        const suffix = this.mode === 'test' ? `-${this.branch}-${this.mode}` : '';
        return `${this.pkgJson.name}${suffix}`.replace(/[^0-9A-Za-z-]/g, '');
    }
    deployProject() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validate();
            const app = yield this.createSeagullApp();
            yield this.customizeStack(app);
            yield app.deployStack();
            const url = (yield aws.getCFURL(this.getAppName(), this.cloudfront)) || '';
            yield new commands_fs_1.FS.WriteFile('/tmp/cfurl.txt', url).execute();
            console.info('cloudfront-url', url);
        });
    }
    diffProject() {
        return __awaiter(this, void 0, void 0, function* () {
            const app = yield this.createSeagullApp();
            yield this.customizeStack(app);
            yield app.diffStack();
        });
    }
    validate() {
        const hasValidProfile = set_aws_credentials_1.setCredsByProfile(this.profile);
        !hasValidProfile && lib.noCredentialsSet();
    }
    getBucketName() {
        return __awaiter(this, void 0, void 0, function* () {
            const accountId = yield aws.getAccountId(this.sts);
            const prefix = `${this.region}-${accountId}-`;
            const bucketName = `${require(`${this.appPath}/package.json`).name}-items`;
            const suffix = `${this.mode !== 'prod' ? `-${this.mode}` : ''}`;
            return `${prefix}${bucketName}${suffix}`;
        });
    }
}
exports.SeagullProject = SeagullProject;
function addResources(appPath, itemsBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const provideAssetFolder = new provide_asset_folder_1.ProvideAssetFolder(appPath);
        yield provideAssetFolder.execute();
        itemsBucket && (yield replaceS3BucketName(appPath, itemsBucket));
    });
}
function replaceS3BucketName(appPath, itemsBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const lambdaPath = `.seagull/deploy/dist/assets/backend/lambda.js`;
        const lambdaFile = `${appPath}/${lambdaPath}`;
        const lambda = yield new commands_fs_1.FS.ReadFile(lambdaFile).execute();
        const lambdaWithBucketName = lambda.replace('demo-bucket', itemsBucket);
        yield new commands_fs_1.FS.WriteFile(lambdaFile, lambdaWithBucketName).execute();
    });
}
function buildCronJson(appPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const cronPath = `${appPath}/dist/cron.json`;
        const cronFile = yield new commands_fs_1.FS.ReadFile(cronPath).execute();
        return cronFile && cronFile !== '' ? JSON.parse(cronFile) : [];
    });
}
//# sourceMappingURL=seagull_project.js.map